searchData={"content_type":"text/markdown","items":[{"doc":"A pure-ish Elixir cryptography tool for the Salsa20 and ChaCha20 stream ciphers.\n\nThis library has a handful of crypto functions that are compatible with NaCl/libsodium\nfor encryption and decryption with shared secret keys. Some sodium-flavored public key\ncryptography functions are also available, and are primarily wrappers around `:crypto` functions.\n\nThe Salsa20/XSalsa20 ciphers are implemented entirely in Elixir while the Poly1305 MAC\nfunction is done through the Erlang `:crypto` module, which is implemented as a NIF with\nOpenSSL bindings.\n\nThe ChaCha20_Poly1305 AEAD cipher is already supported by the `:crypto` module, but XChaCha20\nis not. The HChaCha20 hash function, an intermediate step for generating an XChaCha20 sub-key,\nis implemented in Elixir so `:crypto.crypto_one_time_aead/7` can be leveraged for XChaCha20_Poly1305.\n\nThe ChaCha20/XChaCha20 ciphers do also have pure Elixir implementations just like Salsa20/XSalsa20,\nbut unless you are concerned with long-running NIFs blocking schedulers, you should prefer to use\nthe versions that fully leverage `:crypto` NIFs, which is the behavior of functions in this module.\nIf you wish to use the elixir implementations, you can call them directly with the functions\navailable in `Salchicha.Chacha` ending in `_pure`.\n\nWhile this module contains everything you'll need to encrypt and decrypt with XSalsa20_Poly1305\nand XChaCha20_Poly1305, the internal modules `Salchicha.Salsa` and `Salchicha.Chacha` expose a\nfew additional functions including some primitives and non-extended Salsa20 and ChaCha20 ciphers.","ref":"Salchicha.html","title":"Salchicha","type":"module"},{"doc":"Assume we have a key, and an extended nonce, and a plaintext message\n```elixir\nkey = Salchicha.generate_secret_key()\nnonce = Salchicha.generate_nonce()\nmessage = \"Hello, World!\"\n```","ref":"Salchicha.html#module-examples","title":"Examples - Salchicha","type":"module"},{"doc":"```elixir\nencrypted_message =\n  message\n  |> Salchicha.secretbox(nonce, key)\n  |> IO.iodata_to_binary()\n# <<211, 79, 12, ...>>\n\ndecrypted_message =\n  encrypted_message\n  |> Salchicha.secretbox_open(nonce, key)\n  |> IO.iodata_to_binary()\n# \"Hello, World!\"\n```\n\nThe secretbox'd message is in the format `| --- 16-byte tag --- | --- cipher text --- |`\n\nNote the `IO.iodata_to_binary/1` calls are optional. The input messages can be `t:iodata/0`.","ref":"Salchicha.html#module-xsalsa20-poly1305-via-secretbox-3-and-secretbox_open-3","title":"XSalsa20 Poly1305 via `secretbox/3` and `secretbox_open/3` - Salchicha","type":"module"},{"doc":"```elixir\nencrypted_message =\n  message\n  |> Salchicha.xchacha20_poly1305_encrypt(nonce, key, _aad = \"XCHACHA\")\n  |> IO.iodata_to_binary()\n# <<82, 26, 161, ...>>\n\ndecrypted_message =\n  encrypted_message\n  |> Salchicha.xchacha20_poly1305_decrypt(nonce, key, _aad = \"XCHACHA\")\n# \"Hello, World!\"\n```\n\nThe combined mode encrypted message is in the format `| --- cipher text --- | --- 16-byte tag --- |`\n\nThe AAD is optional and will default to `<<>>`, a zero-length binary.","ref":"Salchicha.html#module-xchacha20-poly1305-in-combined-mode","title":"XChaCha20 Poly1305 in combined mode - Salchicha","type":"module"},{"doc":"```elixir\n{cipher_text, tag} =\n  message\n  |> Salchicha.xchacha20_poly1305_encrypt_detached(nonce, key, _aad = \"XCHACHA\")\n# {<<82, 26, 161, ...>>, <<1, 199, 251, ...>>}\n\ndecrypted_message =\n  cipher_text\n  |> Salchicha.xchacha20_poly1305_decrypt_detached(nonce, key, _aad = \"XCHACHA\", tag)\n# \"Hello, World!\"\n```\n\nDetached mode means the cipher text and tag are returned separately instead of being concatenated together.","ref":"Salchicha.html#module-xchacha20-poly1305-in-detached-mode","title":"XChaCha20 Poly1305 in detached mode - Salchicha","type":"module"},{"doc":"```elixir\n{alice_public, alice_private} = Salchicha.generate_box_keypair()\n{bob_public, bob_private} = Salchicha.generate_box_keypair()\n\n{shared_secret, encrypted_message} = Salchicha.box(message, nonce, bob_public, alice_private)\n# {<<192, 50, 31, ...>>, [<<89, 217, 187, ...>>, ...]}\n\n{^shared_secret, decrypted_message_by_keypair} =\n  Salchicha.box_open(encrypted_message, nonce, alice_public, bob_private)\n\nIO.iodata_to_binary(decrypted_message_by_keypair)\n# \"Hello, World!\"\n\ndecrypted_message_by_shared_secret =\n  encrypted_message\n  |> Salchicha.secretbox_open(nonce, shared_secret)\n  |> IO.iodata_to_binary()\n# \"Hello, World!\"\n```\n\nThe `shared_secret` output key can be used with `secretbox/3` and `secretbox_open/3` for subsequent\nmessages to avoid computing the shared secret from the public and private keys each time.","ref":"Salchicha.html#module-curve25519-xsalsa20-poly1305-via-box-4-and-box_open-4","title":"Curve25519 XSalsa20 Poly1305 via `box/4` and `box_open/4` - Salchicha","type":"module"},{"doc":"```elixir\n{public_key, private_key} = Salchicha.generate_sign_keypair()\n\nsignature = Salchicha.sign(message, private_key)\n# <<51, 62, 180, ...>>\n\nSalchicha.signature_valid?(message, signature, public_key)\n# true\n\nSalchicha.signature_valid?(message <> \"~~~\", signature, public_key)\n# false\n```","ref":"Salchicha.html#module-ed25519-message-signatures","title":"Ed25519 Message Signatures - Salchicha","type":"module"},{"doc":"Additional authenticated data","ref":"Salchicha.html#t:aad/0","title":"Salchicha.aad/0","type":"type"},{"doc":"Encrypts a message with your private key and the recipient's public key using Curve25519_XSalsa20_Poly1305.\n\nThis function behaves like `crypto_box()` and `crypto_box_easy()` do in NaCl and libsodium.","ref":"Salchicha.html#box/4","title":"Salchicha.box/4","type":"function"},{"doc":"- `message` - Plaintext message to be encrypted\n  - `nonce` - 24-byte extended nonce\n  - `their_public` - Recipient's 32-byte public X25519 key\n  - `your_private` - Your 32-byte private X25519 key\n\nReturns a tuple with the computed shared secret key and the encrypted message.\n\nThe returned shared secret key can be used with `secretbox/3` for subsequent encrypted messages to the same recipient\nto avoid having to re-compute the key with `box/4` for each message. Doing this would be similar to using\n`crypto_box_afternm()` in NaCl, which uses the pre-computed shared secret for multiple messages.\n\nCalls `compute_shared_secret/2` then calls `secretbox/3` with the result.","ref":"Salchicha.html#box/4-parameters","title":"Parameters - Salchicha.box/4","type":"function"},{"doc":"Decrypts a message with your private key and the sender's public key using Curve25519_XSalsa20_Poly1305.\n\nThis function behaves like `crypto_box_open()` and `crypto_box_open_easy()` do in NaCl and libsodium.","ref":"Salchicha.html#box_open/4","title":"Salchicha.box_open/4","type":"function"},{"doc":"- `message` - The encrypted message\n  - `nonce` - 24-byte extended nonce\n  - `their_public` - Sender's 32-byte public X25519 key\n  - `your_private` - Your 32-byte private X25519 key\n\nReturns a tuple with the computed shared secret key and the decrypted plaintext.\n\nThe returned shared secret key can be used with `secretbox_open/3` for subsequent messages from the same sender\nto avoid having to re-compute the key with `box_open/4` for each message. Doing this would be similar to using\n`crypto_box_open_afternm()` in NaCl, which uses the pre-computed shared secret for multiple messages.\n\nCalls `compute_shared_secret/2` then calls `secretbox_open/3` with the result.","ref":"Salchicha.html#box_open/4-parameters","title":"Parameters - Salchicha.box_open/4","type":"function"},{"doc":"12-byte nonce used by the ChaCha20 (IETF) cipher","ref":"Salchicha.html#t:chacha_nonce/0","title":"Salchicha.chacha_nonce/0","type":"type"},{"doc":"Encrypted plaintext","ref":"Salchicha.html#t:cipher_text/0","title":"Salchicha.cipher_text/0","type":"type"},{"doc":"Computes the shared secret key given a peer's X25519 public key and your X25519 private key.\n\nSimilar to NaCl/libsodium `crypto_box_beforenm()`/`crypto_box_open_beforenm()`.\n\n_Calls `:crypto.compute_key/4` then calls `Salchicha.Salsa.hsalsa20/2` with the resultant key and 16 zeroes as the input vector._","ref":"Salchicha.html#compute_shared_secret/2","title":"Salchicha.compute_shared_secret/2","type":"function"},{"doc":"Encrypted message to decrypt; `t:cipher_text/0` appended or prepended with `t:tag/0`","ref":"Salchicha.html#t:encrypted_message/0","title":"Salchicha.encrypted_message/0","type":"type"},{"doc":"24-byte extended nonce used by the XSalsa20 and XChaCha20 ciphers","ref":"Salchicha.html#t:extended_nonce/0","title":"Salchicha.extended_nonce/0","type":"type"},{"doc":"Generates a random X25519 key pair for public-key encryption (box/4 and box_open/4).\n\nIf a private key is provided it will recover the public key instead of generating a random key pair.\n\nBehaves like NaCl/libsodium `crypto_box_keypair()`. _Calls `:crypto.generate_key/3`_","ref":"Salchicha.html#generate_box_keypair/1","title":"Salchicha.generate_box_keypair/1","type":"function"},{"doc":"Generates a random 24-byte extended nonce\n\nXSalsa20 and XChaCha20 use a 24-byte nonce, up from the 8 and 8/12 byte nonces\nof the respective Salsa20 and ChaCha20 ciphers.\n\nYou should never reuse the same nonce for a given secret key. 24 bytes are said to be large enough\nto generate nonces randomly - doing so would be ill-advised with 8-byte nonces since collision would\nbe much more likely.","ref":"Salchicha.html#generate_nonce/0","title":"Salchicha.generate_nonce/0","type":"function"},{"doc":"Generates a random 32-byte key","ref":"Salchicha.html#generate_secret_key/0","title":"Salchicha.generate_secret_key/0","type":"function"},{"doc":"Generates a random Ed25519 key pair for message signing and signature verification.\n\nIf a private key is provided it will recover the public key instead of generating a random key pair.\n\nBehaves like NaCl/libsodium `crypto_sign_keypair()`. _Calls `:crypto.generate_key/3`_","ref":"Salchicha.html#generate_sign_keypair/1","title":"Salchicha.generate_sign_keypair/1","type":"function"},{"doc":"Plaintext message to encrypt","ref":"Salchicha.html#t:message/0","title":"Salchicha.message/0","type":"type"},{"doc":"32-byte private key of a Curve25519 (X25519 or Ed25519) key pair","ref":"Salchicha.html#t:private_key/0","title":"Salchicha.private_key/0","type":"type"},{"doc":"32-byte public key of a Curve25519 (X25519 or Ed25519) key pair","ref":"Salchicha.html#t:public_key/0","title":"Salchicha.public_key/0","type":"type"},{"doc":"8-byte nonce used by the Salsa20 cipher","ref":"Salchicha.html#t:salsa_nonce/0","title":"Salchicha.salsa_nonce/0","type":"type"},{"doc":"32-byte shared secret key used by all variations of Salsa/ChaCha","ref":"Salchicha.html#t:secret_key/0","title":"Salchicha.secret_key/0","type":"type"},{"doc":"Encrypts a message with a secret key using the XSalsa20_Poly1305 authenticated cipher.\n\nThis function behaves like `crypto_secretbox()` does in NaCl.","ref":"Salchicha.html#secretbox/3","title":"Salchicha.secretbox/3","type":"function"},{"doc":"- `message` - Plaintext message to be encrypted\n  - `nonce` - 24-byte extended nonce\n  - `key` - 32-byte secret key\n\nThe return value is the cipher text *prepended* by the 16-byte tag (MAC), compatible with NaCl.\n\nReturns an `t:iolist/0` to reduce binary copies. Call `IO.iodata_to_binary/1` if you need a single binary.\n\n_Calls `Salchicha.Salsa.xsalsa20_poly1305_encrypt/3` then concatenates the tag and cipher text_","ref":"Salchicha.html#secretbox/3-parameters","title":"Parameters - Salchicha.secretbox/3","type":"function"},{"doc":"Decrypts a message that was encrypted with `secretbox/3` using the XSalsa20_Poly1305 authenticated cipher.\n\nThis function behaves like `crypto_secretbox_open()` does in NaCl.","ref":"Salchicha.html#secretbox_open/3","title":"Salchicha.secretbox_open/3","type":"function"},{"doc":"- `message` - The encrypted message (tag *prepended* to cipher text)\n  - `nonce` - 24-byte extended nonce\n  - `key` - 32-byte secret key\n\nThe return value is the decrypted plaintext (as an iolist) or `:error` if authentication failed.\n\nReturns an `t:iolist/0` to reduce binary copies. Call `IO.iodata_to_binary/1` if you need the message as a binary.\n\n_Splits tag and cipher text then calls `Salchicha.Salsa.xsalsa20_poly1305_decrypt/4`_","ref":"Salchicha.html#secretbox_open/3-parameters","title":"Parameters - Salchicha.secretbox_open/3","type":"function"},{"doc":"Sign a message with an Ed25519 private key\n\nBehaves like NaCl/libsodium `crypto_sign()`. _Calls `:crypto.sign/4`_","ref":"Salchicha.html#sign/2","title":"Salchicha.sign/2","type":"function"},{"doc":"64-byte Ed25519 message signature","ref":"Salchicha.html#t:signature/0","title":"Salchicha.signature/0","type":"type"},{"doc":"Verifies a message signature with the signer's Ed25519 public key\n\nBehaves like NaCl/libsodium `crypto_sign_open()`. _Calls `:crypto.sign/4`_","ref":"Salchicha.html#signature_valid?/3","title":"Salchicha.signature_valid?/3","type":"function"},{"doc":"Tag or MAC (message authentication code)","ref":"Salchicha.html#t:tag/0","title":"Salchicha.tag/0","type":"type"},{"doc":"Decrypts a message that was encrypted in \"combined mode\" using the XChaCha20_Poly1305 AEAD cipher.\n\nThis function behaves like `crypto_aead_xchacha20poly1305_ietf_decrypt()` does in libsodium.","ref":"Salchicha.html#xchacha20_poly1305_decrypt/4","title":"Salchicha.xchacha20_poly1305_decrypt/4","type":"function"},{"doc":"- `message` - The encrypted message (tag *appended* to cipher text)\n  - `nonce` - 24-byte extended nonce\n  - `key` - 32-byte secret key\n  - `aad` - Additional authenticated data (defaults to `<<>>` i.e. no AAD)\n\nThe return value is the decrypted plaintext as a binary or `:error` if authentication failed.\n\n_Splits cipher text and tag then calls `Salchicha.Chacha.xchacha20_poly1305_decrypt/5`_","ref":"Salchicha.html#xchacha20_poly1305_decrypt/4-parameters","title":"Parameters - Salchicha.xchacha20_poly1305_decrypt/4","type":"function"},{"doc":"Decrypts a message that was encrypted in \"detacheded mode\" using the XChaCha20_Poly1305 AEAD cipher.\n\nThis function behaves like `crypto_aead_xchacha20poly1305_ietf_decrypt_detached()` does in libsodium.","ref":"Salchicha.html#xchacha20_poly1305_decrypt_detached/5","title":"Salchicha.xchacha20_poly1305_decrypt_detached/5","type":"function"},{"doc":"- `cipher_text` - The encrypted message (only the cipher text, not appended with the tag)\n  - `nonce` - 24-byte extended nonce\n  - `key` - 32-byte secret key\n  - `aad` - Additional authenticated data (defaults to `<<>>` i.e. no AAD)\n  - `tag` - 16-byte Poly1305 authentication tag or MAC\n\nThe return value is the decrypted plaintext as a binary or `:error` if authentication failed.\n\nThis function differs from `xchacha20_poly1305_decrypt/4` by returning the tag and cipher text separately\n\nThis \"detached mode\" function differs from the \"combined mode\" `xchacha20_poly1305_decrypt/4` in that\nthe cipher text and tag are supplied as separate parameters, not combined as a single message.\n\n_Calls `Salchicha.Chacha.xchacha20_poly1305_decrypt/5`_","ref":"Salchicha.html#xchacha20_poly1305_decrypt_detached/5-parameters","title":"Parameters - Salchicha.xchacha20_poly1305_decrypt_detached/5","type":"function"},{"doc":"Encrypts a message with a secret key using the XChaCha20_Poly1305 AEAD cipher in \"combined mode\".\n\nThis function behaves like `crypto_aead_xchacha20poly1305_ietf_encrypt()` does in libsodium.","ref":"Salchicha.html#xchacha20_poly1305_encrypt/4","title":"Salchicha.xchacha20_poly1305_encrypt/4","type":"function"},{"doc":"- `message` - Plaintext message to be encrypted\n  - `nonce` - 24-byte extended nonce\n  - `key` - 32-byte secret key\n  - `aad` - Additional authenticated data (defaults to `<<>>` i.e. no AAD)\n\nThe return value is the cipher text *appended* by the 16-byte tag (MAC), i.e. \"combined mode\".\n\nReturns an `t:iolist/0` to reduce binary copies. Call `IO.iodata_to_binary/1` if you need a single binary.\n\n_Calls `Salchicha.Chacha.xchacha20_poly1305_encrypt/4` then concatenates the cipher text and tag_","ref":"Salchicha.html#xchacha20_poly1305_encrypt/4-parameters","title":"Parameters - Salchicha.xchacha20_poly1305_encrypt/4","type":"function"},{"doc":"Encrypts a message with a secret key using the XChaCha20_Poly1305 AEAD cipher in \"detached mode\".\n\nThis function behaves like `crypto_aead_xchacha20poly1305_ietf_encrypt_detached()` does in libsodium.","ref":"Salchicha.html#xchacha20_poly1305_encrypt_detached/4","title":"Salchicha.xchacha20_poly1305_encrypt_detached/4","type":"function"},{"doc":"- See `xchacha20_poly1305_encrypt/4`\n\nThis \"detached mode\" function differs from the \"combined mode\" `xchacha20_poly1305_encrypt/4`\nby returning the tag and cipher text separately in a tuple in the form `{cipher_text, tag}`.\nBoth `cipher_text` and `tag` will already be binaries.\n\n_Calls `Salchicha.Chacha.xchacha20_poly1305_encrypt/4`_","ref":"Salchicha.html#xchacha20_poly1305_encrypt_detached/4-parameters","title":"Parameters - Salchicha.xchacha20_poly1305_encrypt_detached/4","type":"function"},{"doc":"Implementation of the ChaCha20 and XChaCha20 Ciphers\n\n>","ref":"Salchicha.Chacha.html","title":"Salchicha.Chacha","type":"module"},{"doc":">\n> This module is documented for completeness and for curious cryptographers.\n> It contains some ChaCha primitives that you likely won't need to use directly.","ref":"Salchicha.Chacha.html#module-internal-module-info","title":"Internal module {: .info} - Salchicha.Chacha","type":"module"},{"doc":"Erlang's `:crypto` module supports the chacha20_poly1305 AEAD stream cipher,\nstandardized by the IETF, as its underlying NIFs are bindings to OpenSSL, which implements it.\n\nAnalogously to Salsa20 and XSalsa20, XChaCha20 is a way to use 192-bit nonces\nwith ChaCha20 by hashing the key and part of the extended nonce to generate a\nsub-key, which is used as the input key for ChaCha20.\n\nTo leverage the crypto module, we had to implement the HChaCha20 hash function\nin elixir to then pass the resulting sub-key and sub-nonce to `:crypto.crypto_one_time_aead/7`.","ref":"Salchicha.Chacha.html#module-purpose","title":"Purpose - Salchicha.Chacha","type":"module"},{"doc":"The HChaCha20 function takes the first 16 bytes of the extended 24-byte XChaCha20 nonce,\nexpands the key and the 16-byte nonce slice into a block in place of the block counter and\nusual smaller nonce. That block has 20 rounds of mutation, and instead of summing the block\nwith its starting state as is done with keystream generation, 8 of the 16 32-bit words are taken\nand used as the sub-key, which is the input key for the chacha20 cipher. The sub-nonce is the latter\n8 bytes of the extended nonce prepended with 4 zero bytes for the 12-byte nonce that the IETF version\nof ChaCha20 specifies.\n\nAfter running HChaCha20, we have the inputs required use the `:crypto` module's `:chacha20_poly1305`\nfunctionality in the capacity of XChaCha20_Poly1305. This is all in service of leveraging the performance\nbenefits of the crypto NIFs, which are necessarily going to be more performant than anything implemented\nin pure elixir/erlang like the `:kcl` package.\n\nFor reference, I also implemented the ChaCha20/XChaCha20 functions in Elixir that don't use `crypto_one_time_aead/7`,\nonly leveraging `:crypto.mac/3` for the Poly1305 MAC similarly to `Salchicha.Salsa`. The only reason you might prefer\nto use these functions (ending in `_pure`) over the NIF ones is that if you have an exceptionally large message\nthe long-running NIF would not yield to the erlang scheduler and could block other processes. In most cases\nyou should prefer the impure variants - the `Salchicha` functions will use them by default.\n\n*ChaCha20 is a variant of the Salsa20 cipher. I will discuss in greater detail the implementation\nin the `Salchicha.Salsa` module, where much is applicable here.*\n\nReferences for Salsa family of ciphers\n- https://cr.yp.to/snuffle/spec.pdf\n- https://cr.yp.to/chacha/chacha-20080128.pdf\n- https://cr.yp.to/snuffle/xsalsa-20110204.pdf\n- https://datatracker.ietf.org/doc/html/rfc7539\n- https://datatracker.ietf.org/doc/html/draft-irtf-cfrg-xchacha","ref":"Salchicha.Chacha.html#module-implementation","title":"Implementation - Salchicha.Chacha","type":"module"},{"doc":"After the XChaCha20 sub-key is generated in elixir, the crypto NIF function performs the\nheavy lifting. Performance should be speedy.","ref":"Salchicha.Chacha.html#module-performance-considerations","title":"Performance considerations - Salchicha.Chacha","type":"module"},{"doc":"","ref":"Salchicha.Chacha.html#chacha20_poly1305_decrypt_pure/5","title":"Salchicha.Chacha.chacha20_poly1305_decrypt_pure/5","type":"function"},{"doc":"","ref":"Salchicha.Chacha.html#chacha20_poly1305_encrypt_pure/4","title":"Salchicha.Chacha.chacha20_poly1305_encrypt_pure/4","type":"function"},{"doc":"Plain ChaCha20 Cipher. Crypto primitive.\n\nXOR a message (encrypt or decrypt) with ChaCha20.\nThis uses 12 byte nonces and isn't authenticated with Poly1305 MAC.\n\nThe IETF standardized version of ChaCha20 uses 12 byte nonces and a 4 byte block\ncounter instead of 8 byte nonces and an 8 byte block counter.\nThe IETF standard's 4 byte (32 bit) block counter means messages are limited to 256GB:\n2^32 blocks (4 GigaBlocks) * 64 bytes per block = 256GB.\n\nBecause of how the block state is arranged in the cipher, if you take an 8-byte nonce\nand prepend it with 4 zeros to form a 12 byte nonce, this is equivalent to\nthe pre-IETF-standardized version of ChaCha20 that uses 8 byte nonces. In fact,\nthe XChaCha20 sub-nonce derived from the 24-byte extended nonce is simply the last 8 byte\nprepended with 4 zeros.\n\nBlock counter starts at 0 by default.\n\nImplemented in Elixir.\n\nEquivalent NIF-powered version of this would be\n```elixir\n# Encrypt\n:crypto.crypto_one_time(:chacha20, key, iv, plaintext, _encrypt = true)\n\n# Decrypt\n:crypto.crypto_one_time(:chacha20, key, iv, ciphertext, _encrypt = false)\n```\n\nSome things worth noting from the above usage of the `:crypto` module function:\n  - The initialization vector (iv) arg is NOT simply the nonce - it the full 16-byte user-controlled input.\n    - For an 8 or 12 byte nonce, you'd have to prepend your initial block counter in little-endian.\n      - `iv = <<_block_counter=0::little-32>> <> my_twelve_byte_nonce`\n      - `iv = <<_block_counter=0::little-64>> <> my_eight_byte_nonce`\n    - How annoying!\n  - The fifth argument, the boolean `encrypt` flag, is irrelevant.\n    - This is an unauthenticated stream cipher where the input message is XOR'd with the keystream.\n      - Encryption and decryption are fundamentally the same operation.\n      - While the flag is required, whether it's `true` or `false` makes no difference.\n    - How annoying!","ref":"Salchicha.Chacha.html#chacha20_xor/4","title":"Salchicha.Chacha.chacha20_xor/4","type":"function"},{"doc":"HChaCha20 hash function for deriving a sub-key for XChaCha20. Crypto primitive.\n\nSecond arg can be extended nonce (first 16 bytes of the nonce will be used) or 16 bytes.","ref":"Salchicha.Chacha.html#hchacha20/2","title":"Salchicha.Chacha.hchacha20/2","type":"function"},{"doc":"","ref":"Salchicha.Chacha.html#xchacha20_poly1305_decrypt/5","title":"Salchicha.Chacha.xchacha20_poly1305_decrypt/5","type":"function"},{"doc":"","ref":"Salchicha.Chacha.html#xchacha20_poly1305_decrypt_pure/5","title":"Salchicha.Chacha.xchacha20_poly1305_decrypt_pure/5","type":"function"},{"doc":"","ref":"Salchicha.Chacha.html#xchacha20_poly1305_encrypt/4","title":"Salchicha.Chacha.xchacha20_poly1305_encrypt/4","type":"function"},{"doc":"","ref":"Salchicha.Chacha.html#xchacha20_poly1305_encrypt_pure/4","title":"Salchicha.Chacha.xchacha20_poly1305_encrypt_pure/4","type":"function"},{"doc":"Implementation of the Salsa20 and XSalsa20 Ciphers\n\n>","ref":"Salchicha.Salsa.html","title":"Salchicha.Salsa","type":"module"},{"doc":">\n> This module is documented for completeness and for curious cryptographers.\n> It contains some Salsa primitives that you likely won't need to use directly.","ref":"Salchicha.Salsa.html#module-internal-module-info","title":"Internal module {: .info} - Salchicha.Salsa","type":"module"},{"doc":"To support xsalsa20_poly1305 without a NIF, we have to implement the\nSalsa20 cipher and HSalsa20 hash function to use 192-bit nonces in the capacity\nof XSalsa20.\n\nAlong with leveraging the `:crypto` module to perform the poly1305 MAC function\nand xor'ing arbitrary-length binaries, by being more thoughtful and explicit\nwith our implementation we should be able to eek out better performance\nthan the `:kcl` package provides.","ref":"Salchicha.Salsa.html#module-purpose","title":"Purpose - Salchicha.Salsa","type":"module"},{"doc":"The `:kcl` package is an impressive pure-elixir NaCl/libsodium compatible library I've used\nin the past for xsalsa20_poly1305 encryption.\n\nSome of the key differences in our implementation compared to Kcl\n- Heavy use of explicit binary pattern matching instead of more traditional implicit enumeration\n- Intermediate block state stored in a 16-element tuple that is mutated during the 20-round hot loop instead of lists\n- Minimized the number binary copies, returning iolists when appropriate, instead of concatenating binaries\n- XOR whole keystream and message blocks instead of XOR'ing one byte at a time\n- Poly1305 MAC handled by `:crypto` module instead of implemented in elixir\n- Curve25519 public-key crypto functionality handled by `:crypto` module\n\nAdditionally there appears to be a bug in how Kcl serializes the 16-byte block counter during key expansion:\nAccording to the spec it's supposed to be little endian, and it happens to be for blocks 0-255, but for larger\nblock counts, Kcl would be incompatible with NaCl/libsodium-type libraries.\n\nThe XOR'ing of the keystream and message occurs one block (64 bytes) at a time using `:crypto.exor/2`, which\nis implemented as a NIF, but this could have just as easily been done in elixir by casting the\nkeystream and message block binaries into 512-bit integers, passing them to `Bitwise.bxor/2`, and casting the result\ninto a binary.\n\nThe cipher functions were implemented in the order they're defined in the original Salsa specification,\nand though it's using a lot of explicit binary pattern matching, it turned out to be quite legible.\nIn a single statement of binary pattern matching, the 512-bit initial block state is cast into 16\nlittle-endian 32-bit words. Standard elixir patterns might have you iterate through the binary until the\nend was reached, but matching and casting all sixteen block elements in a single statement then returning\na tuple is explicit, clear, and simple to understand when referencing the spec.\n\nReaders interested in cryptography are encouraged to read more about the Salsa20/ChaCha20 ciphers.\n\nReferences for Salsa family of ciphers\n- https://cr.yp.to/snuffle/spec.pdf\n- https://cr.yp.to/chacha/chacha-20080128.pdf\n- https://cr.yp.to/snuffle/xsalsa-20110204.pdf\n- https://datatracker.ietf.org/doc/html/rfc7539\n- https://datatracker.ietf.org/doc/html/draft-irtf-cfrg-xchacha","ref":"Salchicha.Salsa.html#module-implementation","title":"Implementation - Salchicha.Salsa","type":"module"},{"doc":"The entire keystream generation and xor'ing the message with the stream is done in elixir,\nonly performing the Poly1305 MAC function through the `:crypto` module. Although it was implemented\nas thoughtfully and explicitly as possible with memory usage and performance in mind, using any\nof the Salsa modes will likely be less performant than ChaCha.","ref":"Salchicha.Salsa.html#module-performance-considerations","title":"Performance considerations - Salchicha.Salsa","type":"module"},{"doc":"HSalsa20 hash function for deriving a sub-key for XSalsa20. Crypto primitive.\n\nSecond arg can be extended nonce (first 16 bytes of the nonce will be used) or 16 bytes.","ref":"Salchicha.Salsa.html#hsalsa20/2","title":"Salchicha.Salsa.hsalsa20/2","type":"function"},{"doc":"","ref":"Salchicha.Salsa.html#salsa20_poly1305_decrypt/4","title":"Salchicha.Salsa.salsa20_poly1305_decrypt/4","type":"function"},{"doc":"","ref":"Salchicha.Salsa.html#salsa20_poly1305_encrypt/3","title":"Salchicha.Salsa.salsa20_poly1305_encrypt/3","type":"function"},{"doc":"Plain Salsa20 Cipher. Crypto primitive.\n\nXOR a message (encrypt or decrypt) with Salsa20.\nThis uses 8 byte nonces and isn't authenticated with Poly1305 MAC.\nBlock counter starts at 0 by default.","ref":"Salchicha.Salsa.html#salsa20_xor/4","title":"Salchicha.Salsa.salsa20_xor/4","type":"function"},{"doc":"","ref":"Salchicha.Salsa.html#xsalsa20_poly1305_decrypt/4","title":"Salchicha.Salsa.xsalsa20_poly1305_decrypt/4","type":"function"},{"doc":"","ref":"Salchicha.Salsa.html#xsalsa20_poly1305_encrypt/3","title":"Salchicha.Salsa.xsalsa20_poly1305_encrypt/3","type":"function"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"* Add Changelog\n  * Add NaCl public-key encryption via X25519 ECDH\n    * `Salchicha.box/4`\n    * `Salchicha.box_open/4`\n    * `Salchicha.generate_box_keypair/0`\n    * `Salchicha.compute_shared_secret/2`\n  * Add NaCl public-key message signing via Ed25519 EdDSA\n    * `Salchicha.sign/2`\n    * `Salchicha.signature_valid?/3`\n    * `Salchicha.generate_sign_keypair/0`\n  * Add examples of public key crypto usage","ref":"changelog.html#v0-4-0-2025-03-12","title":"v0.4.0 (2025-03-12) - Changelog","type":"extras"},{"doc":"* Improve docs\n    * Add examples for XSalsa20/XChaCha20 encryption/decryption\n    * Add additional type docs\n    * Change references of \"block count\" to \"block counter\" for clarity\n    * Fix typos\n  * Internal function `Salchicha.Salsa.xsalsa20_poly1305_decrypt/4` takes `cipher_text` and `tag` as arguments instead of combined message. `Salchicha.secretbox_open/3` remains unchanged.\n  * Expose Salsa20 Poly1305 functions\n    * `Salchicha.Salsa.salsa20_poly1305_encrypt/3`\n    * `Salchicha.Salsa.salsa20_poly1305_decrypt/4`","ref":"changelog.html#v0-3-0-2025-03-07","title":"v0.3.0 (2025-03-07) - Changelog","type":"extras"},{"doc":"* Improve docs and typespecs\n  * Add pure elixir implementations of ChaCha20/XChaCha20 Poly1305 functions in `Salchicha.Chacha`\n    * `Salchicha.Chacha.chacha20_poly1305_encrypt_pure/4`\n    * `Salchicha.Chacha.chacha20_poly1305_decrypt_pure/5`\n    * `Salchicha.Chacha.xchacha20_poly1305_encrypt_pure/4`\n    * `Salchicha.Chacha.xchacha20_poly1305_decrypt_pure/5`\n    * Add `Salchicha.Chacha.chacha20_xor/4` stream cipher primitive\n  * Internal `Salchicha.Salsa` improvements\n    * Verify MAC before decrypting remaining ciphertext","ref":"changelog.html#v0-2-0-2025-03-06","title":"v0.2.0 (2025-03-06) - Changelog","type":"extras"},{"doc":"* Initial release\n  * Support XSalsa20 Poly1305\n    * `Salchicha.secretbox/3`\n    * `Salchicha.secretbox_open/3`\n    * Internal functions and primitives in `Salchicha.Salsa`\n  * Support XChaCha20 Poly1305 in combined and detached modes\n    * `Salchicha.xchacha20_poly1305_encrypt/4`\n    * `Salchicha.xchacha20_poly1305_decrypt/4`\n    * `Salchicha.xchacha20_poly1305_encrypt_detached/4`\n    * `Salchicha.xchacha20_poly1305_decrypt_detached/5`\n    * Internal functions and primitives in `Salchicha.Chacha`","ref":"changelog.html#v0-1-0-2025-02-19","title":"v0.1.0 (2025-02-19) - Changelog","type":"extras"}],"producer":{"name":"ex_doc","version":"0.37.1"},"proglang":"elixir"}